package io.tswf.gpt.mistral.tokenization.client

import feign.okhttp.OkHttpClient
import groovy.transform.CompileStatic
import groovy.transform.Memoized
import io.tswf.gpt.mistral.tokenization.api.ApiClient
import io.tswf.gpt.mistral.tokenization.api.client.TokenizationApi
import okhttp3.Credentials

@CompileStatic
class MistralTokenizationApi {

    String mistralTokenizationBaseUrl = System.findGlobalProperty("mistral-tokenization.api.base-url").orNull()

    Proxy mistralTokenizationProxy = null

    String mistralTokenizationProxyUser = null

    String mistralTokenizationProxyPassword = null

    MistralTokenizationApi(@DelegatesTo(MistralTokenizationApi) Closure configurator = {  }) {
        if (configurator != null) {
            configurator.delegate = this
            configurator.call(this)
        }
    }

    @Memoized
    TokenizationApi getTokenizationApi() {
        return apiClientsBuilder.buildClient(TokenizationApi)
    }

    @Memoized
    ApiClient getApiClientsBuilder() {
        return new ApiClient().tap {
            feignBuilder.tap {
                client(
                    new OkHttpClient(
                        new okhttp3.OkHttpClient.Builder().tap {
                            mistralTokenizationProxy?.let {
                                proxy(it)
                                // Авторизация для прокси, если требуется
                                if (mistralTokenizationProxyUser.isNotNullOrBlank()) {
                                    if (mistralTokenizationProxyPassword == null) {
                                        throw new IllegalStateException("Password was not given for proxy user with username: '${mistralTokenizationProxyUser}'")
                                    }

                                    if (mistralTokenizationProxy == null) {
                                        throw new IllegalStateException("Proxy password set when proxy is null!")
                                    }

                                    if (mistralTokenizationProxy.type() != Proxy.Type.HTTP) {
                                        throw new IllegalStateException("Proxy authorization is only supported for HTTP proxy type, not for '${mistralTokenizationProxy.type()?.name()}'!")
                                    }

                                    proxyAuthenticator { route, response ->
                                        response.request().newBuilder()
                                                .header("Proxy-Authorization", Credentials.basic(mistralTokenizationProxyUser, mistralTokenizationProxyPassword))
                                                .build()
                                    }
                                }
                            }
                        }
                        .build()
                    )
                )
            }

            if (mistralTokenizationBaseUrl != null) {
                basePath = mistralTokenizationBaseUrl
            }
        }
    }
}
